import sys
from socket import gaierror, create_connection
from ssl import SSLContext, SSLError
from typing import List
import urllib.request
import json
from .protocol import DefinedProtocol

_configurations = []


def load_recommendations() -> List:
    protocols = []

    with urllib.request.urlopen(  # nosec
            "https://ssl-config.mozilla.org/guidelines"  # nosec
            "/latest.json") as url:  # nosec
        data = json.loads(url.read().decode())
        for configuration, level in data['configurations'].items():
            for version in level['tls_versions']:
                if version not in _configurations:
                    protocols.append(DefinedProtocol(version, configuration))
                    _configurations.append(version)
    return protocols


class HighTls:

    def __init__(self, domain, slide=False):
        self.domain = domain
        self.slide = slide
        if self._check_protocols(load_recommendations()):
            sys.exit(1)

    def _check_protocols(self, protocols: List) -> int:
        exit_code = 0
        for protocol in protocols:
            print("\n-------\nProtocol: {}\nShould be active: {}"
                  .format(protocol.protocol, protocol.is_allowed()))
            try:
                with create_connection((self.domain, 443)) as sock:
                    ctx = SSLContext(protocol.get_ssl_attribute())
                    for opposite in protocol \
                            .get_opposite_ssl_as_op_no(_configurations):
                        ctx.options |= opposite
                    with ctx.wrap_socket(sock, server_hostname=self.domain) \
                            as context_sock:
                        print("Connected with: {}"
                              .format(context_sock.version()))
                        print("Using cipher: {}"
                              .format(context_sock.cipher()))
                        failed = False
            except gaierror:
                print("{} is not valid".format(self.domain))
                return 1
            except (ValueError, SSLError):
                failed = True

            if (failed and protocol.is_allowed()) or \
                    (not failed and not protocol.is_allowed()):
                print("\033[91m\tWrong configuration\033[0m")
                exit_code = 1

        print("\nTest on {} \033[91mfailed\033[0m".format(
            self.domain) if exit_code == 1 else "\033[94mpassed\033[0m")
        if self.slide and exit_code:
            print(
                "But we'll let it slide for now"
                "(you are testing an appspot domain)")
        return exit_code if not self.slide else 0



import subprocess  # nosec
import sys


class NoSsl:

    def __init__(self, domain):
        self.domain = domain
        if not self.test_process():
            sys.exit(1)

    def test_process(self):
        ossl = subprocess.run(  # nosec
            ['openssl', 's_client', '-connect', '{}:443'.format(self.domain), '-ssl3'],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE, timeout=15)

        if str(ossl.stderr).find("Option unknown option -ssl3") != -1:
            print("openssl does not support SSLv3")
            return False

        if str(ossl.stderr).find("no protocols available") != -1:
            print("no protocols available")
            return False

        if ossl.returncode == 1:
            print('No sign of SSLv3 allowance found')
            return True

        print('SSLv3 seems to be allowed')
        return False



import requests
import sys


class CorsPolicy:

    def __init__(self, domain):
        self.domain = domain
        if not self._scan():
            sys.exit(1)

    def _scan(self):
        res = requests.get('https://{}'.format(self.domain), timeout=20,
                           allow_redirects=False)
        res_head = res.headers.get('Access-Control-Allow-Origin')

        if not res_head:
            print("Failing policy test: No Allowed Origins Specified")
            return False
        if str(res_head) == '*':
            print("Failing policy test: No Specific Origins Specified")
            return False

        print("Passing policy test: Allowed Origins Specified")
        return True



from .cors_policy import CorsPolicy
from .high_tls import HighTls
from .hsts import Hsts
from .no_http import NoHttp
from .no_ssl import NoSsl

__all__ = ['CorsPolicy', 'HighTls', 'Hsts', 'NoHttp', 'NoSsl']



import ssl
from typing import List


class DefinedProtocol:

    def __init__(self, protocol: str, configuration: str):
        self.protocol = protocol
        self.configuration = configuration

    def get_ssl_attribute(self):
        try:
            return getattr(ssl, f"PROTOCOL_{self.protocol.replace('.', '_')}")
        except AttributeError:
            if self.is_allowed:
                return ssl.PROTOCOL_TLS

    def is_allowed(self) -> bool:
        return self.configuration in ['modern', 'intermediate']

    def get_opposite_ssl_as_op_no(self, versions) -> List:
        opposite_versions = []
        for version in versions:
            if not version == self.protocol:
                opposite_versions \
                    .append(getattr(ssl, f"OP_NO_{version.replace('.', '_')}"))

        return opposite_versions



import os

from setuptools import setup, find_packages

with open('requirements.txt') as f:
    install_requires = f.read().splitlines()

setup(
    name='sec-helpers',
    packages=find_packages(),
    include_package_data=True,
    version=os.getenv('TAG_NAME', '0.0.0'),
    license='gpl-3.0',
    description='DAST Security Helpers',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='VWT Digital',
    author_email='support@vwt.digital',
    url='https://github.com/vwt-digital/sec-helpers/tree/master',
    keywords=['DAST', 'security', 'helpers', 'vwt'],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Natural Language :: English',
    ],
    install_requires=install_requires,
    python_requires='>=3.6',
)



import requests
import sys


class NoHttp:

    def __init__(self, domain):
        self.domain = domain
        if self._get_protocol_enabled():
            print(
                "Failing http status check: http is enabled or not"
                "redirecting properly to https")
            sys.exit(1)
        if not self._get_protocol_enabled("https"):
            print("Failing https status check: https is not enabled")
            sys.exit(1)
        print(
            "Successful http status check: http is disabled or"
            "redirects to https")

    def _get_protocol_enabled(self, url_pre: str = "http") -> bool:
        """
        For a given domain, return a bool to signal if expected http behaviour
        is implemented.

        Expected behaviour is that http should be disabled, or 304-redirecting
        to https://

        Parameters:
        domain (string): A DNS domain, like e.g. dev.corp.example
        url_pre (string): Category of protocol (secure or not secure)

        Returns:
        bool: True or False

        """
        try:
            print(
                "Starting GET request to {}://{}".format(url_pre, self.domain))
            r = requests\
                .get('{}://{}'.format(url_pre, self.domain),
                     timeout=60, allow_redirects=False)
        except Exception as e:
            print("Exception connecting to {}://{} with {}".format(url_pre,
                                                                   self.domain,
                                                                   str(e)))
            return not url_pre == 'http'
        else:
            print("GET request to {}://{} returned status {}"
                  .format(url_pre,
                          self.domain,
                          r.status_code))
            return not (r.status_code == 301 or (
                    r.status_code == 302 and url_pre == 'http'))



import requests
import sys
import re


class Hsts:

    def __init__(self, domain, age=10368000):
        self.domain = domain
        self.age = age
        if not self._get_hsts_enabled():
            print(
                "Failing HSTS status check: no"
                "HSTS header or max-age too short")
            sys.exit(1)

        print("Successful HSTS status check")

    def _get_hsts_enabled(self) -> bool:
        """
        For a given domain, return a bool to signal if expected HSTS header is
        present

        Expected behaviour is that HSTS should be set, with max-age of at least
        10368000 seconds (120 days), or other seconds if specified

        Returns:
        bool: True or False

        """
        try:
            print("Starting GET request to http://{}".format(self.domain))
            r = requests.get('https://{}'.format(self.domain), timeout=60,
                             allow_redirects=False)
        except Exception as e:
            print(
                "Exception connecting to https://{} with {}"
                .format(self.domain, str(e)))
            return False
        else:
            print(
                "Strict-Transport-Security header on https://{} returned {}"
                .format(self.domain, r.headers.get(
                        'Strict-Transport-Security')))
            m = re.search(r'max-age=(\d+)',
                          r.headers.get('Strict-Transport-Security', ''))
            max_age = -1
            if m:
                max_age = int(m.group(1))
            return max_age >= self.age
